import subprocess
import tempfile
import shutil


def capture(command):
    proc = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    out, err = proc.communicate()
    return out, err, proc.returncode


def test_extract_metrics_folder_not_found_exception_handling():
    config_dirpath = tempfile.mkdtemp()
    _, err, returncode = capture(
        [
            "msgram",
            "extract",
            "-o",
            "sonarqube",
            "-ep",
            config_dirpath,
            "-dp",
            "sonar-output-fake",
        ]
    )

    assert returncode == 1
    message = "src.cli.exceptions.exceptions.MeasureSoftGramCLIException: No files .json found inside folder."
    assert message in err.decode("utf-8")
    shutil.rmtree(config_dirpath)


def test_extract_metrics_config_folder_not_found_exception_handling():
    msg, _, returncode = capture(
        [
            "msgram",
            "extract",
            "-o",
            "sonarqube",
            "-ep",
            "config-fake",
            "-dp",
            "sonar-output-fake",
        ]
    )

    assert returncode == 1
    assert "FileNotFoundError: extract directory" in msg.decode("utf-8")



from src.cli.resources.subcharacteristic import get_subcharacteristic_value

from resources import calculate_characteristics as core_calculate


def get_characteristic_value(chars, tsqmi):
    char_calculated = []
    for char in tsqmi:
        char_key = char["key"]

        char_calculated.append(
            {
                "key": char_key,
                "value": {m["key"]: m["value"] for m in chars}[char_key],
                "weight": char["weight"],
            }
        )

    return char_calculated


def calculate_characteristics(config, subchars):
    characteristics = config["characteristics"]
    calculate_infos = []

    for char in characteristics:
        calculate_infos.append(
            {
                "key": char["key"],
                "subcharacteristics": get_subcharacteristic_value(
                    subchars, char["subcharacteristics"]
                ),
            }
        )

    headers = ["Id", "Name", "Description", "Value", "Created at"]
    return core_calculate({"characteristics": calculate_infos}), headers



import sys
import pytest
import tempfile
import shutil
import os
import copy

from io import StringIO
from pathlib import Path

from src.cli.commands.cmd_extract import get_infos_from_name, command_extract

EXTRACT_ARGS = {
    "output_origin": "sonarqube",
    "extracted_path": Path(""),
    "data_path": Path(""),
    "language_extension": "py",
}


def test_get_file_infos():
    file_path = "tests/unit/data/fga-eps-mds-2022-2-MeasureSoftGram-CLI-01-11-2023-21-59-03-develop.json"

    file_name = get_infos_from_name(file_path)
    assert (
        "fga-eps-mds-2022-2-MeasureSoftGram-CLI-01-11-2023-21-59-03-develop-extracted.msgram"
        in file_name
    )


def test_not_get_file_infos_wrong_name():
    filename = "metrics/wrong-name.json"

    with pytest.raises(SystemExit) as e:
        _ = get_infos_from_name(filename)

    assert e.value.code == 1


def test_command_extract_should_succeed():
    config_dirpath = tempfile.mkdtemp()
    extract_dirpath = tempfile.mkdtemp()

    shutil.copy("tests/unit/data/msgram.json", f"{config_dirpath}/msgram.json")

    shutil.copy(
        "tests/unit/data/fga-eps-mds-2022-2-MeasureSoftGram-CLI-01-11-2023-21-59-03-develop.json",
        f"{extract_dirpath}/fga-eps-mds-2022-2-MeasureSoftGram-CLI-01-11-2023-21-59-03-develop.json",
    )

    args = {
        "output_origin": "sonarqube",
        "extracted_path": Path(config_dirpath),
        "data_path": Path(extract_dirpath),
        "language_extension": "py",
    }

    captured_output = StringIO()
    sys.stdout = captured_output

    command_extract(args)

    sys.stdout = sys.__stdout__

    assert "Metrics successfully extracted" in captured_output.getvalue()
    assert os.path.isfile(
        f"{config_dirpath}/fga-eps-mds-2022-2-MeasureSoftGram-"
        "CLI-01-11-2023-21-59-03-develop-extracted.msgram"
    )

    shutil.rmtree(config_dirpath)
    shutil.rmtree(extract_dirpath)


@pytest.mark.parametrize(
    "extract_arg",
    ["output_origin", "extracted_path", "language_extension"],
)
def test_extract_invalid_args(extract_arg):
    captured_output = StringIO()
    sys.stdout = captured_output

    args = copy.deepcopy(EXTRACT_ARGS)
    del args[extract_arg]

    with pytest.raises(SystemExit):
        command_extract(args)

    sys.stdout = sys.__stdout__
    assert (
        f"KeyError: args['{extract_arg}'] - non-existent parameters"
        in captured_output.getvalue()
    )


def test_extract_fail_no_dp_or_rep():
    extract_dirpath = tempfile.mkdtemp()
    args = {
        "output_origin": "sonarqube",
        "language_extension": "py",
        "extracted_path": Path(extract_dirpath),
    }

    captured_output = StringIO()
    sys.stdout = captured_output
    with pytest.raises(SystemExit):
        command_extract(args)

    sys.stdout = sys.__stdout__

    assert (
        "It is necessary to pass the data_path or repository_path parameters"
        in captured_output.getvalue()
    )


def test_extract_directory_not_exist():
    args = {
        "output_origin": "sonarqube",
        "language_extension": "py",
        "extracted_path": Path("tests/directory_not_exist"),
        "data_path": Path("tests/directory_not_exist"),
    }

    captured_output = StringIO()
    sys.stdout = captured_output
    with pytest.raises(SystemExit):
        command_extract(args)

    sys.stdout = sys.__stdout__

    assert "FileNotFoundError: extract directory" in captured_output.getvalue()



import sys
import logging
from datetime import datetime
from rich.logging import RichHandler


LOG_LEVELS = {
    "DEBUG": logging.DEBUG,
    "INFO": logging.INFO,
    "WARNING": logging.WARNING,
    "ERROR": logging.ERROR,
    "CRITICAL": logging.CRITICAL,
}

LOG_FORMATS = {
    "RICH": "%(module)-12s: [line: %(lineno)-3s] %(message)s",
    "DEBUG": "%(levelname)-8s %(module)-12s %(funcName) -15s : %(lineno)-3s %(message)s",
    "INFO": "%(levelname)-8s: %(message)s",
    "WARNING": "%(levelname)-8s: %(module)-8s:%(lineno)-3s %(message)s",
    "ERROR": "%(asctime)s: %(levelname)-8s: %(funcName)s :%(lineno)d %(message)s",
    "CRITICAL": "%(asctime)-15s %(levelname)-8s %(filename)-15s %(module)-8s:%(lineno)-3s %(message)s",
}

# =================================================================================================

now = datetime.now()
date = now.strftime("%y%m%d")
datefmt = "%d %b %Y - %H:%M:%S"  # 30 Dec 2022 11:41:57
file_name = "src/logs/debug.log"


def config_logger(log_mod):
    if log_mod == "quiet":
        basic_config("INFO", "WARNING", "w")
    elif log_mod == "verbose":
        basic_config("DEBUG", "DEBUG", "w")
    else:
        basic_config("WARNING", "ERROR", "w")


# =================================================================================================


def basic_config(console_level_name, file_level_name, file_mode):
    console_handler = logging.StreamHandler(sys.stdout)
    console_formatter = logging.Formatter(LOG_FORMATS[console_level_name], datefmt)
    console_handler.setFormatter(console_formatter)
    # console_handler.setLevel(LOG_LEVELS[console_level_name])

    # file_handler = logging.FileHandler(file_name, file_mode)
    # file_formatter = logging.Formatter(LOG_FORMATS[file_level_name])
    # file_handler.setFormatter(file_formatter)
    # file_handler.setLevel(LOG_LEVELS[file_level_name])

    rich_handler = RichHandler(level=logging.DEBUG, rich_tracebacks=True)
    rich_handler.setFormatter(logging.Formatter(LOG_FORMATS["RICH"], datefmt))

    logger = logging.getLogger("msgram")
    logger.addHandler(console_handler)
    # logger.addHandler(rich_handler)
    # logger.addHandler(file_handler)
    logger.setLevel(LOG_LEVELS[console_level_name])

    logger_console = logging.getLogger("console")
    logger_console.addHandler(console_handler)
    logger_console.setLevel(LOG_LEVELS[console_level_name])

    logger_file = logging.getLogger("file")
    # logger_file.addHandler(file_handler)
    logger_file.addHandler(console_handler)
    logger_file.setLevel(LOG_LEVELS[file_level_name])


# =================================================================================================
# logging testing

# logger.log(logging.DEBUG, "This is an debug message.")
# logger.log(logging.INFO, "This is an info message.")
# logger.log(logging.ERROR, "This is an error message.")
# logger.log(logging.WARNING, "This is an warning message.")
# logger.log(logging.CRITICAL, "This is an critical message.")
# logger.error("[bold red blink]This is an error custom message[/]", extra={"markup": True})


# =================================================================================================



from pathlib import Path

from src.cli.resources.characteristic import calculate_characteristics
from src.cli.jsonReader import open_json_file


def test_calculate_characteristics():
    config = open_json_file(Path("tests/unit/data/msgram.json"))
    subcharacteristics = [
        {"key": "testing_status", "value": 0.8633460569923477},
        {"key": "modifiability", "value": 0.650528195701257},
    ]

    infos, headers = calculate_characteristics(config, subcharacteristics)

    assert headers == ["Id", "Name", "Description", "Value", "Created at"]
    assert infos == {
        "characteristics": [
            {"key": "reliability", "value": 0.8633460569923477},
            {"key": "maintainability", "value": 0.650528195701257},
        ]
    }



import json
import math
from pathlib import Path

import rich.progress
from rich import print

from src.cli.exceptions import exceptions

REQUIRED_SONAR_JSON_KEYS = ["paging", "baseComponent", "components"]
REQUIRED_TRK_MEASURES = ["test_failures", "test_errors", "files", "ncloc"]
REQUIRED_SONAR_BASE_COMPONENT_KEYS = [
    "id",
    "key",
    "name",
    "qualifier",
    "measures",
]


def file_reader(path_file):
    json_data = open_json_file(path_file, True)

    check_sonar_format(json_data)
    check_metrics_values(json_data)

    components = json_data["components"]
    components.append(json_data["baseComponent"])
    return components


def read_multiple_files(directory: Path, pattern: str):
    for path_file in directory.glob(f"*.{pattern}"):
        try:
            yield open_json_file(path_file), path_file.name
        except exceptions.MeasureSoftGramCLIException:
            print(
                f"[red]Error calculating {path_file.name}: Failed to decode the JSON file.\n"
            )


def folder_reader(dir_path, pattern):
    num_files_error = 0
    if not list(dir_path.glob(f"*.{pattern}")):
        raise exceptions.MeasureSoftGramCLIException(
            f"No files .{pattern} found inside folder."
        )

    for path_file in dir_path.glob(f"*.{pattern}"):
        try:
            yield file_reader(path_file), path_file.name, num_files_error
            num_files_error = 0
        except exceptions.MeasureSoftGramCLIException as e:
            print(f"[green]Reading:[/] [black]{path_file.name}[/]")
            print(f"[red]Error  : {e}\n")
            num_files_error += 1


def open_json_file(path_file: Path, disable=False):
    try:
        with rich.progress.open(
            path_file,
            "rb",
            description=path_file.name,
            disable=disable,
            style="bar.back",
            complete_style="bar.complete",
            finished_style="bar.finished",
            pulse_style="bar.pulse",
        ) as file:
            return json.load(file)

    except FileNotFoundError:
        raise exceptions.FileNotFound("The file was not found")
    except IsADirectoryError:
        raise exceptions.UnableToOpenFile(f"File {path_file.name} is a directory")
    except json.JSONDecodeError as error:
        raise exceptions.InvalidMetricsJsonFile(
            f"Failed to decode the JSON file. {error}"
        )


def get_missing_keys_str(attrs, required_attrs):
    missing_keys = [req_key for req_key in required_attrs if req_key not in attrs]
    return ", ".join(missing_keys)


def check_sonar_format(json_data):
    attributes = list(json_data.keys())
    missing_keys = get_missing_keys_str(attributes, REQUIRED_SONAR_JSON_KEYS)

    if len(missing_keys) > 0:
        raise exceptions.InvalidMetricsJsonFile(
            f"Invalid Sonar JSON keys. Missing keys are: {missing_keys}"
        )

    base_component = json_data["baseComponent"]
    base_component_attrs = list(base_component.keys())
    missing_keys = get_missing_keys_str(
        base_component_attrs, REQUIRED_SONAR_BASE_COMPONENT_KEYS
    )

    if len(missing_keys) > 0:
        raise exceptions.InvalidMetricsJsonFile(
            f"Invalid Sonar baseComponent keys. Missing keys are: {missing_keys}"
        )

    base_component_measures = base_component["measures"]
    base_component_measures_attrs = [bc["metric"] for bc in base_component_measures]
    missing_keys = get_missing_keys_str(
        base_component_measures_attrs, REQUIRED_TRK_MEASURES
    )

    if len(missing_keys) > 0:
        raise exceptions.InvalidMetricsJsonFile(
            f"Invalid Sonar baseComponent TRK measures. Missing keys are: {missing_keys}"
        )

    if len(json_data["components"]) == 0:
        raise exceptions.InvalidMetricsJsonFile(
            "File with valid schema but no metrics data."
        )


def check_file_extension(file_name):
    if file_name.split(".")[-1] != "json":
        raise exceptions.InvalidMetricsJsonFile("Only JSON files are accepted.")


def raise_invalid_metric(key, metric):
    raise exceptions.InvalidMetricException(
        f'Invalid metric value in "{key}" component for the "{metric}" metric'
    )


def check_metrics_values(json_data):
    try:
        for component in json_data["components"]:
            for measure in component["measures"]:
                value = measure["value"]

                try:
                    if value is None or math.isnan(float(value)):
                        raise_invalid_metric(component["key"], measure["metric"])
                except (ValueError, TypeError):
                    raise_invalid_metric(component["key"], measure["metric"])
    except KeyError as e:
        raise exceptions.InvalidMetricsJsonFile(
            "Failed to validate Sonar JSON metrics. Please check if the file is a valid Sonar JSON"
        ) from e


def validate_metrics_post(response_status):
    if 200 <= response_status <= 299:
        return "OK: Metrics uploaded successfully"

    return f"FAIL: The host service server returned a {response_status} error. Trying again"



from .metrics import *  # noqa: F403, F401
from .measure import *  # noqa: F403, F401
from .subcharacteristic import *  # noqa: F403, F401
from .characteristic import *  # noqa: F403, F401
from .tsqmi import *  # noqa: F403, F401



import os
import json

from src.cli.utils import print_error, print_info

metrics = {}
metrics["sonar"] = [
    "tests",
    "test_failures",
    "test_errors",
    "coverage",
    "test_execution_time",
    "functions",
    "complexity",
    "comment_lines_density",
    "duplicated_lines_density",
]

metrics["github"] = [
    "resolved_issues",
    "total_issues",
    "sum_ci_feedback_times",
    "total_builds",
]

measures = {}
measures["sonar"] = [
    "passed_tests",
    "test_builds",
    "test_errors",
    "test_coverage",
    "non_complex_file_density",
    "commented_file_density",
    "duplication_absense",
]

measures["github"] = ["team_throughput", "ci_feedback_time"]


def should_process_sonar_metrics(config):
    for characteristic in config.get("characteristics", []):
        for subcharacteristic in characteristic.get("subcharacteristics", []):
            for measure in subcharacteristic.get("measures", []):
                if measure.get("key") in measures["sonar"]:
                    return True
    return False


def should_process_github_metrics(config):
    for characteristic in config.get("characteristics", []):
        for subcharacteristic in characteristic.get("subcharacteristics", []):
            for measure in subcharacteristic.get("measures", []):
                if measure.get("key") in measures["github"]:
                    return True
    return False


def read_msgram(file_path):
    try:
        with open(file_path, "r") as file:
            return json.load(file)
    except IsADirectoryError as e:
        print_error(f"> [red] Error: {e}")
        return False
    except FileNotFoundError as e:
        print_error(f"> [red] Error: {e}")
        return False


def list_msgram_files(folder_path):
    try:
        if not os.path.isdir(folder_path):
            raise NotADirectoryError(f"{folder_path} is not a directory.")

        msgram_files = [
            file for file in os.listdir(folder_path) if file.endswith(".msgram")
        ]
        return msgram_files

    except NotADirectoryError as e:
        print_error(f"> [red] Error: {e}")
        return False


def save_metrics(file_name, metrics):
    directory = os.path.dirname(file_name)

    os.makedirs(directory, exist_ok=True)

    output_file_path = os.path.join(
        directory, os.path.basename(file_name).replace(".msgram", ".metrics")
    )
    with open(output_file_path, "w") as output_file:
        json.dump(metrics, output_file, indent=2)

    print_info(f"> [blue] Metrics saved to: {output_file_path}\n")


def process_sonar_metrics(folder_path, msgram_files, github_files):
    processed_files = []

    for file in msgram_files:
        if file not in github_files:
            print_info(f"> [blue] Processing {file}")
            sonar_metrics_dict = read_msgram(os.path.join(folder_path, file))

            if not sonar_metrics_dict:
                print_error(f"> [red] Error to read sonar metrics in: {folder_path}\n")
                return False

            processed_files.append((file, sonar_metrics_dict))

    return processed_files


def process_github_metrics(folder_path, github_files, metrics):
    if not github_files:
        print_error(f"> [red] GitHub files not found in the directory: {folder_path}\n")
        return False

    print_info(f"> [blue] GitHub metrics found in: {folder_path}\n")

    first_github_file = read_msgram(os.path.join(folder_path, github_files[0]))

    if not first_github_file:
        print_error(f"> [red] Error to read github metrics in: {folder_path}\n")
        return False

    github_key = next(iter(first_github_file.keys() - metrics["sonar"]), "")

    github_metrics = [
        {
            "metric": metric,
            "value": next(
                (
                    m["value"]
                    for m in first_github_file[github_key]
                    if m["metric"] == metric
                ),
                None,
            ),
        }
        for metric in metrics["github"]
    ]

    return (github_files[0], github_metrics)


def aggregate_metrics(folder_path, config: json):
    msgram_files = list_msgram_files(folder_path)

    if not msgram_files:
        print_error("> [red]Error: Can not read msgram files in provided directory")
        return False

    github_files = [file for file in msgram_files if file.startswith("github_")]

    file_content = {}

    github_metrics = []

    have_metrics = False

    if should_process_github_metrics(config):
        file, github_metrics = process_github_metrics(
            folder_path, github_files, metrics
        )

        if not github_metrics:
            print_error("> [red]Error: Unexpected result from process_github_metrics")
            return False

        have_metrics = True

    if should_process_sonar_metrics(config):
        result = process_sonar_metrics(folder_path, msgram_files, github_files)

        if not result or len(result) != 1:
            print_error("> [red]Error: Unexpected result from process_sonar_metrics")
            return False

        have_metrics = True
        file, file_content = result[0]

    if not have_metrics:
        print_error("> [red]Error: No metrics where found in the .msgram files")
        return False

    file_content["github_metrics"] = github_metrics

    save_metrics(os.path.join(folder_path, file), file_content)

    return True



import copy
import os
import shutil
import sys
import tempfile
from io import StringIO
from pathlib import Path
from unittest.mock import patch

import pytest

from src.cli.commands.cmd_calculate import calculate_all, command_calculate, show_tree
from src.cli.jsonReader import open_json_file
from staticfiles import DEFAULT_PRE_CONFIG as pre_config

CALCULATE_ARGS = {
    "output_format": "csv",
    "config_path": Path(""),
    "extracted_path": Path(""),
}


def test_show_tree(capfd):

    data_calculated = {
        'repository': [{'key': 'repository', 'value': 'fga-eps-mds-2022-2-MeasureSoftGram-CLI'}],
        'version': [{'key': 'version', 'value': '01-05-2023-21-40'}],
        'measures': [
            {'key': 'passed_tests', 'value': 1.0},
            {'key': 'test_builds', 'value': 0.9996066627522133},
            {'key': 'test_coverage', 'value': 0.40234848484848484},
            {'key': 'non_complex_file_density', 'value': 0.44347274991556906},
            {'key': 'commented_file_density', 'value': 0.04318181818181818},
            {'key': 'duplication_absence', 'value': 1.0},
            {'key': 'team_throughput', 'value': 0.6969696969696971},
            {'key': 'ci_feedback_time', 'value': 0.06117908787541713}
        ],
        'subcharacteristics': [
            {'key': 'testing_status', 'value': 0.8421061048464034},
            {'key': 'maturity', 'value': 0.06117908787541713},
            {'key': 'modifiability', 'value': 0.6415437113263573},
            {'key': 'functional_completeness', 'value': 0.6969696969696971}
        ],
        'characteristics': [
            {'key': 'reliability', 'value': 0.5970282960684735},
            {'key': 'maintainability', 'value': 0.6415437113263573},
            {'key': 'functional_suitability', 'value': 0.6969696969696971}
        ],
        'tsqmi': [{'key': 'tsqmi', 'value': 0.6455181338484177}]
    }

    expected_output = (
        "Overview - tree:\n\n"
        "\n"
        "tsqmi: 0.6455181338484177\n"
        "├── reliability: 0.5970282960684735\n"
        "│   ├── testing_status 0.8421061048464034\n"
        "│   │   ├── passed_tests 1.0\n"
        "│   │   ├── test_builds 0.9996066627522133\n"
        "│   │   └── test_coverage 0.40234848484848484\n"
        "│   └── maturity 0.06117908787541713\n"
        "│       └── ci_feedback_time 0.06117908787541713\n"
        "├── maintainability: 0.6415437113263573\n"
        "│   └── modifiability 0.6415437113263573\n"
        "│       ├── non_complex_file_density 0.44347274991556906\n"
        "│       └── commented_file_density 0.04318181818181818\n"
        "└── functional_suitability: 0.6969696969696971\n"
        "    └── functional_completeness 0.6969696969696971\n"
        "        └── team_throughput 0.6969696969696971"
    )

    show_tree(data_calculated, pre_config)

    captured = capfd.readouterr()

    assert captured.out.strip() == expected_output.strip()


@pytest.mark.parametrize(
    "calculate_arg", ["output_format", "config_path", "extracted_path"]
)
def test_calculate_invalid_args(calculate_arg):
    captured_output = StringIO()
    sys.stdout = captured_output

    args = copy.deepcopy(CALCULATE_ARGS)
    del args[calculate_arg]

    with pytest.raises(SystemExit):
        command_calculate(args)

    sys.stdout = sys.__stdout__
    assert (
        f"KeyError: args['{calculate_arg}'] - non-existent parameters"
        in captured_output.getvalue()
    )


@pytest.mark.parametrize(
    "output_format,multiple_files",
    [
        ("tabular", False),
        ("tree", False),
        ("raw", False),
        ("csv", True),
        ("json", True),
    ],
)
def test_calculate_file(output_format, multiple_files):
    config_dirpath = tempfile.mkdtemp()
    extract_dirpath = tempfile.mkdtemp()

    shutil.copy("tests/unit/data/msgram.json", f"{config_dirpath}/msgram.json")

    extracted_file_name = "fga-eps-mds-2022-2-MeasureSoftGram-CLI-01-05-2023-21-40-30-develop-extracted.metrics"
    shutil.copy(
        f"tests/unit/data/{extracted_file_name}",
        f"{extract_dirpath}/{extracted_file_name}",
    )

    args = {
        "output_format": output_format,
        "config_path": Path(config_dirpath),
        "extracted_path": Path(
            extract_dirpath + (f"/{extracted_file_name}" if not multiple_files else "")
        ),
    }
    if not multiple_files:
        calculate_patch = patch("builtins.input", return_value=output_format)
        calculate_patch.start()

    command_calculate(args)

    assert len(os.listdir(config_dirpath)) == 1
    assert len(os.listdir(extract_dirpath)) == 1

    shutil.rmtree(config_dirpath)
    shutil.rmtree(extract_dirpath)


def test_calculate_all_dict():
    file_name = "fga-eps-mds-2022-2-MeasureSoftGram-CLI-01-05-2023-21-40-30-develop-extracted.metrics"
    json_data = open_json_file(Path(f"tests/unit/data/{file_name}"))
    config = open_json_file(Path("tests/unit/data/msgram.json"))

    calculated_result = calculate_all(json_data, file_name, config)
    calculate_expected = {
        "repository": [
            {"key": "repository", "value": "fga-eps-mds-2022-2-MeasureSoftGram-CLI"}
        ],
        "version": [{"key": "version", "value": "01-05-2023-21-40"}],
        "measures": [
            {"key": "passed_tests", "value": 1.0},
            {
                "key": "test_builds",
                "value": 0.9996066627522133,
            },
            {
                "key": "test_coverage",
                "value": 0.40234848484848484,
            },
            {
                "key": "non_complex_file_density",
                "value": 0.44347274991556906,
            },
            {
                "key": "commented_file_density",
                "value": 0.04318181818181818,
            },
            {"key": "duplication_absense", "value": 1.0},
        ],
        "subcharacteristics": [
            {
                "key": "testing_status",
                "value": 0.8421061048464034,
            },
            {
                "key": "modifiability",
                "value": 0.6415437113263573,
            },
        ],
        "characteristics": [
            {
                "key": "reliability",
                "value": 0.8421061048464034,
            },
            {
                "key": "maintainability",
                "value": 0.6415437113263573,
            },
        ],
        "tsqmi": [
            {
                "key": "tsqmi",
                "value": 0.7485723162667646,
            }
        ],
    }
    assert calculated_result.get("repository") == calculate_expected.get("repository")
    assert calculated_result.get("version") == calculate_expected.get("version")
    measures_result = calculated_result.get("measures")
    measures_expected = calculate_expected.get("measures")
    for measure_result, measure_expected in zip(measures_result, measures_expected):
        assert measure_result.get("key") == measure_expected.get("key")
        assert pytest.approx(measure_result.get("value")) == measure_expected.get(
            "value"
        )

    subcharacteristics_result = calculated_result.get("subcharacteristics")
    subcharacteristics_expected = calculate_expected.get("subcharacteristics")
    for subcharacteristic_result, subcharacteristic_expected in zip(
        subcharacteristics_result, subcharacteristics_expected
    ):
        assert subcharacteristic_result.get("key") == subcharacteristic_expected.get(
            "key"
        )
        assert pytest.approx(
            subcharacteristic_result.get("value")
        ) == subcharacteristic_expected.get("value")

    characteristics_result = calculated_result.get("characteristics")
    characteristics_expected = calculate_expected.get("characteristics")
    for characteristic_result, characteristic_expected in zip(
        characteristics_result, characteristics_expected
    ):
        assert characteristic_result.get("key") == characteristic_expected.get("key")
        assert pytest.approx(
            characteristic_result.get("value")
        ) == characteristic_expected.get("value")

    tsqmi_result = calculated_result.get("tsqmi")[0]
    tsqmi_expected = calculate_expected.get("tsqmi")[0]
    assert tsqmi_result.get("key") == tsqmi_expected.get("key")
    assert pytest.approx(tsqmi_result.get("value")) == tsqmi_expected.get("value")


def test_calculate_invalid_config_file():
    captured_output = StringIO()
    sys.stdout = captured_output

    config_dirpath = tempfile.mkdtemp()

    shutil.copy("tests/unit/data/invalid_json.json", f"{config_dirpath}/msgram.json")

    args = {
        "output_format": "csv",
        "config_path": Path(config_dirpath),
        "extracted_path": Path("."),
    }

    with pytest.raises(SystemExit):
        command_calculate(args)

    sys.stdout = sys.__stdout__
    assert (
        f"Error reading msgram.json config file in {config_dirpath}"
        in captured_output.getvalue()
    )

    shutil.rmtree(config_dirpath)


def test_calculate_invalid_extracted_file():
    captured_output = StringIO()
    sys.stdout = captured_output

    config_dirpath = tempfile.mkdtemp()
    extract_dirpath = tempfile.mkdtemp()

    shutil.copy("tests/unit/data/msgram.json", f"{config_dirpath}/msgram.json")

    extracted_file_name = "invalid_json.json"
    shutil.copy(
        f"tests/unit/data/{extracted_file_name}",
        f"{extract_dirpath}/{extracted_file_name}",
    )

    args = {
        "output_format": "csv",
        "config_path": Path(config_dirpath),
        "extracted_path": Path(extract_dirpath + f"/{extracted_file_name}"),
    }

    command_calculate(args)

    sys.stdout = sys.__stdout__
    assert (
        f"Error calculating {extract_dirpath}/{extracted_file_name}"
        in captured_output.getvalue()
    )
    assert "All calculations performed" not in captured_output.getvalue()

    shutil.rmtree(config_dirpath)
    shutil.rmtree(extract_dirpath)



def get_metric_value(extracted):
    """
    Função que recupera o valor mais recente da métrica
    """
    # Métricas que o parâmetro é uma lista de valores
    listed_values = [
        "coverage",
        "complexity",
        "functions",
        "comment_lines_density",
        "duplicated_lines_density",
        "resolved_issues",
        "total_issues",
        "sum_ci_feedback_times",
        "total_builds",
    ]

    # Métricas que o parâmetro é extraido do UTS
    uts_values = ["test_execution_time", "tests"]
    response_data = {}

    # Para todos os arquivos extraidos
    for path_readed in extracted.values():
        # Para cada métrica dentro das medidas
        for metric in path_readed:
            metric_name = metric["metric"]
            metric_value = metric["value"]

            # Se ela for "agregada", olho apenas arquivos FIL,
            # se não, salvo somente a última.
            all_value_list = listed_values + uts_values
            if metric_name in all_value_list:
                response_data.setdefault(metric_name, []).append(metric_value)
            else:
                response_data[metric_name] = metric_value

    return response_data



import json
import logging
import sys
from pathlib import Path

from rich.console import Console
from rich.prompt import Confirm
from staticfiles import DEFAULT_PRE_CONFIG

from src.cli.utils import print_error, print_info, print_panel, print_rule
from src.config.settings import FILE_CONFIG

logger = logging.getLogger("msgram")


def command_init(args):
    try:
        config_path: Path = args["config_path"]

    except Exception as e:
        logger.error(f"KeyError: args[{e}] - non-existent parameters")
        print_error(f"KeyError: args[{e}] - non-existent parameters")
        sys.exit(1)

    logger.debug(config_path)
    file_path = config_path / FILE_CONFIG

    console = Console()
    console.clear()
    print_rule("MSGram", "[#708090]Init to set config file[/]:")

    if not config_path.exists():
        print_info(f"Created dir: {config_path}")
        config_path.mkdir()

    replace = True

    if file_path.exists():
        print_info(f"MSGram config file [bold red]'{FILE_CONFIG}'[/] exists already!")
        replace = Confirm.ask(
            f"> Do you want to replace [bold blue]'{FILE_CONFIG}'[/]?"
        )

    if replace:
        try:
            with file_path.open("w") as f:
                f.write(json.dumps(DEFAULT_PRE_CONFIG, indent=4))
        except OSError:
            console.line(2)
            print_error("Error opening or writing to file")
        print_info(
            f"The file config: '{config_path.name}/msgram.json' was created successfully."
        )

    else:
        print_info(f"The file config: '{config_path.name}/msgram.json' not changed...")

    print_panel(
        "> [#008080]Run msgram extract -o sonarqube -dp data_path -ep extract_path[/], to extract supported metrics!"
    )



import logging

from rich import box, print
from rich.console import Console
from rich.panel import Panel
from rich.progress import BarColumn, Progress, TaskProgressColumn, TextColumn
from rich.table import Table

logger = logging.getLogger("msgram")
console = Console(highlight=False, soft_wrap=False, width=140)


def print_info(text: str):
    """Print an info message."""
    console.print(text, style="green")


def print_warn(text: str):
    """Print a warning message."""
    console.print(text, style="yellow")


def print_error(text: str):
    """Print a error message."""
    console.print(text, style="red")


def print_table(the_dict: dict, table_name: str = "", field: str = ""):
    table = Table(
        title=table_name,
        title_style="bold",
        row_styles=["none", "dim"],
        border_style="bright_yellow",
        pad_edge=False,
        box=box.MINIMAL,
    )

    table.add_column(
        field,
        no_wrap=True,
        header_style="bold cyan",
        footer_style="bright_cian",
        style="cyan",
    )

    table.add_column(
        "values",
        no_wrap=True,
        header_style="bold red",
        footer_style="bright_red",
        style="red",
    )

    for field, value in the_dict.items():
        table.add_row(str(field), str(value))

    console.print(table)


def make_progress_bar() -> Progress:
    progress_bar = Progress(
        TextColumn("{task.description}"),
        TextColumn("[bold bright_red]Waiting  "),
        BarColumn(complete_style="red"),
        TaskProgressColumn(),
        refresh_per_second=10,
        transient=True,
    )
    return progress_bar


def print_rule(title: str = "", text: str = "", style: str = "#4682B4"):
    if title:
        console.rule(f"{title}", style=style)
    else:
        console.rule(style=style)
    if text:
        console.print(text, style="grey58")

    console.line()


def print_panel(menssage: str, title: str = "Next steps"):
    console.line(2)
    print(
        Panel(
            menssage,
            title=title,
            title_align="center",
            style="#4F4F4F",
            border_style="#A9A9A9",
            padding=(1, 2),
            width=140,
        ),
    )



from src.cli.resources.measure import get_measure_value

from resources import calculate_subcharacteristics as core_calculate


def get_subcharacteristic_value(subchars, char):
    subchar_calculated = []
    for subchar in char:
        subchar_key = subchar["key"]

        subchar_calculated.append(
            {
                "key": subchar_key,
                "value": {m["key"]: m["value"] for m in subchars}.get(
                    subchar_key, None
                ),
                "weight": subchar["weight"],
            }
        )

    return subchar_calculated


def calculate_subcharacteristics(config, measures):
    calculate_infos = []

    for characteristic in config["characteristics"]:
        subcharacteristics = characteristic.get("subcharacteristics", [])
        for subchar in subcharacteristics:
            if measures:
                subchar_info = {
                    "key": subchar["key"],
                    "measures": get_measure_value(measures, subchar["measures"]),
                }
                calculate_infos.append(subchar_info)

    headers = ["Id", "Name", "Description", "Value", "Created at"]
    return core_calculate({"subcharacteristics": calculate_infos}), headers



from pathlib import Path

from src.cli.resources.tsqmi import calculate_tsqmi
from src.cli.jsonReader import open_json_file


def test_calculate_tsqmi():
    config = open_json_file(Path("tests/unit/data/msgram.json"))
    characteristics = [
        {"key": "reliability", "value": 0.8633460569923477},
        {"key": "maintainability", "value": 0.650528195701257},
    ]

    infos, headers = calculate_tsqmi(config, characteristics)

    assert headers == ["Id", "Value", "Created at"]
    assert infos == {"tsqmi": [{"key": "tsqmi", "value": 0.7643799276297641}]}



import pytest
import json
import os
import tempfile

from src.cli.aggregate_metrics import (
    should_process_github_metrics,
    should_process_sonar_metrics,
)
from src.cli.aggregate_metrics import read_msgram, save_metrics
from src.cli.aggregate_metrics import (
    process_github_metrics,
    process_sonar_metrics,
    aggregate_metrics,
)


TEST_DATA_DIR = os.path.join(os.path.dirname(__file__), "data")

with open(os.path.join(TEST_DATA_DIR, "msgram.json"), "r") as file:
    config = json.load(file)

with open(os.path.join(TEST_DATA_DIR, "allmsgram.json"), "r") as file:
    all_config = json.load(file)

with open(os.path.join(TEST_DATA_DIR, "onlygithubmsgram.json"), "r") as file:
    only_github_msgram = json.load(file)


@pytest.mark.parametrize(
    "config, expected_result",
    [
        (config, True),
        (all_config, True),
        (only_github_msgram, False),
        ({}, False),
        ({"characteristics": []}, False),
        ({"characteristics": [{"subcharacteristics": []}]}, False),
        ({"characteristics": [{"subcharacteristics": [{"measures": []}]}]}, False),
    ],
)
def test_should_process_sonar_metrics(config, expected_result):
    result = should_process_sonar_metrics(config)
    assert result == expected_result


@pytest.mark.parametrize(
    "config, expected_result",
    [
        (config, False),
        (all_config, True),
        (only_github_msgram, True),
        ({}, False),
        ({"characteristics": []}, False),
        ({"characteristics": [{"subcharacteristics": []}]}, False),
        ({"characteristics": [{"subcharacteristics": [{"measures": []}]}]}, False),
    ],
)
def test_should_process_github_metrics(config, expected_result):
    result = should_process_github_metrics(config)
    assert result == expected_result


def test_read_msgram():
    # Create a temporary directory for testing
    with tempfile.TemporaryDirectory() as temp_dir:
        file_path = os.path.join(temp_dir, "test_file.msgram")
        expected_result = {"key": "value"}

        # Write the expected content to the temporary file
        with open(file_path, "w") as file:
            json.dump(expected_result, file)

        # Test reading from the temporary file
        assert read_msgram(file_path) == expected_result

        # Test reading from a directory (should return False)
        assert read_msgram(temp_dir) is False

        # Test reading from a nonexistent file (should return False)
        nonexistent_file_path = os.path.join(temp_dir, "nonexistent_file.msgram")
        assert read_msgram(nonexistent_file_path) is False


def test_process_github_metrics():
    result = process_github_metrics(TEST_DATA_DIR, [], {"sonar": [], "github": []})
    assert result is False

    folder_path = TEST_DATA_DIR
    github_file_name = "github_nlohmann-json-19-11-2023-12-53-58-extracted.msgram"

    metrics = {"sonar": ["some_metric"], "github": ["resolved_issues", "total_issues"]}

    result = process_github_metrics(folder_path, [github_file_name], metrics)

    # Validate the result
    expected_result = (
        github_file_name,
        [
            {"metric": "resolved_issues", "value": 25},
            {"metric": "total_issues", "value": 30},
        ],
    )
    assert result == expected_result


def test_process_sonar_metrics():
    with tempfile.TemporaryDirectory() as temp_dir:
        sonar_file_name = "fga-eps-mds-2022-2-MeasureSoftGram-CLI-01-05-2023-21-40-30-develop-extracted.msgram"
        sonar_file_path = os.path.join(temp_dir, sonar_file_name)

        github_file_name = "github_nlohmann-json-19-11-2023-12-53-58-extracted.msgram"
        github_file_path = os.path.join(temp_dir, github_file_name)

        sonar_data = {"sonar_metric": 42}
        github_data = {"github_metrics": [{"metric": "resolved_issues", "value": 25}]}

        with open(sonar_file_path, "w") as sonar_file:
            json.dump(sonar_data, sonar_file)

        with open(github_file_path, "w") as github_file:
            json.dump(github_data, github_file)

        result = process_sonar_metrics(temp_dir, [sonar_file_name], [github_file_name])
        expected_result = [(sonar_file_name, sonar_data)]
        assert result == expected_result

        assert os.path.exists(sonar_file_path)
        assert os.path.exists(github_file_path)


def test_save_metrics():
    with tempfile.TemporaryDirectory() as temp_dir:
        file_name = os.path.join(temp_dir, "test_file.msgram")
        metrics = {"metric1": 10, "metric2": 20}

        save_metrics(file_name, metrics)

        output_file_path = os.path.join(temp_dir, "test_file.metrics")
        assert os.path.exists(output_file_path)

        with open(output_file_path, "r") as output_file:
            saved_metrics = json.load(output_file)

        assert saved_metrics == metrics


def test_aggregate_metrics():
    with tempfile.TemporaryDirectory() as temp_dir:
        folder_path = temp_dir

        msgram_file1 = os.path.join(folder_path, "file1.msgram")
        msgram_file2 = os.path.join(folder_path, "github_file.msgram")

        with open(msgram_file1, "w") as file:
            json.dump({"some_metric": 42}, file)

        with open(msgram_file2, "w") as file:
            json.dump(
                {
                    "github_metrics": [
                        {"metric": "resolved_issues", "value": 25},
                        {"metric": "total_issues", "value": None},
                    ]
                },
                file,
            )

        result = aggregate_metrics(folder_path, all_config)

        assert result is True

        output_file_path = os.path.join(folder_path, "file1.metrics")
        assert os.path.exists(output_file_path)

        with open(output_file_path, "r") as output_file:
            saved_metrics = json.load(output_file)

        expected_metrics = {
            "some_metric": 42,
            "github_metrics": [
                {"metric": "resolved_issues", "value": 25},
                {"metric": "total_issues", "value": None},
                {"metric": "sum_ci_feedback_times", "value": None},
                {"metric": "total_builds", "value": None},
            ],
        }
        assert saved_metrics == expected_metrics



import argparse

from pathlib import Path

from src.cli.commands.cmd_init import command_init
from src.cli.commands.cmd_extract import command_extract
from src.cli.commands.cmd_calculate import command_calculate
from src.cli.commands.cmd_list import command_list

from src.config.settings import (
    AVAILABLE_IMPORTS,
    SUPPORTED_FORMATS,
    DEFAULT_CONFIG_PATH,
    AVAILABLE_GEN_FORMATS,
)


def create_parser():
    parser = argparse.ArgumentParser(
        prog="msgram",
        description="Command line interface for measuresoftgram",
        epilog="Thanks for using %(prog)s!",
    )

    subparsers = parser.add_subparsers(
        title="subcommands",
        dest="command",
        help="sub-command help",
    )

    # =====================================< COMMAND init >=====================================
    parser_init = subparsers.add_parser(
        "init",
        help="Create a init file `.measuresoftgram` with your default organization, product and repositories",
    )

    parser_init.add_argument(
        "-cp",
        "--config_path",
        type=lambda p: Path(p).absolute(),
        default=DEFAULT_CONFIG_PATH,
        help="Path to default config directory",
    )
    parser_init.set_defaults(func=command_init)  # function command init

    # =====================================< COMMAND list >=====================================
    parser_list_config = subparsers.add_parser(
        "list",
        help="Listing configurations parameters.",
    )

    parser_list_config.add_argument(
        "-cp",
        "--config_path",
        type=lambda p: Path(p).absolute(),
        default=DEFAULT_CONFIG_PATH,
        help="Path to default config directory.",
    )

    parser_list_config.add_argument(
        "all",
        nargs="?",
        help="Show configuration file.",
    )

    parser_list_config.set_defaults(func=command_list)  # function command list config

    # =====================================< COMMAND extract >=====================================
    parser_extract = subparsers.add_parser("extract", help="Extract supported metrics")

    parser_extract.add_argument(
        "-o",
        "--output_origin",
        required=True,
        type=str,
        choices=(AVAILABLE_IMPORTS),
        help=(
            "Import a metrics files/repository from some origin. Valid values are: "
            + ", ".join(AVAILABLE_IMPORTS)
        ),
    )

    parser_extract.add_argument(
        "-dp",
        "--data_path",
        type=lambda p: Path(p).absolute(),
        help="Path to analysis data directory",
    )

    parser_extract.add_argument(
        "-ep",
        "--extracted_path",
        type=lambda p: Path(p).absolute(),
        default=DEFAULT_CONFIG_PATH,
        help="Path to the extracted directory",
    )

    parser_extract.add_argument(
        "-le",
        "--language_extension",
        type=str,
        help="The source code language extension",
        default="py",
    )

    parser_extract.add_argument(
        "-rep",
        "--repository_path",
        type=str,
        help="Path to analysis git repository",
    )

    parser_extract.set_defaults(func=command_extract)  # function command extract

    # =====================================< COMMAND calculate >=====================================
    parser_calculate = subparsers.add_parser(
        "calculate",
        help="Calculates all entities",
    )

    parser_calculate.add_argument(
        "all",
        type=str,
        nargs="?",
        help=(
            "Returns the calculated value of the entities: measures, subcharacteristics, characteristics, tsqmi"
        ),
    )

    parser_calculate.add_argument(
        "-ep",
        "--extracted_path",
        type=lambda p: Path(p).absolute(),
        help="Path to the extracted directory",
    )

    parser_calculate.add_argument(
        "-cp",
        "--config_path",
        type=lambda p: Path(p).absolute(),
        default=DEFAULT_CONFIG_PATH,
        help="Path to the config directory",
    )

    parser_calculate.add_argument(
        "-o",
        "--output_format",
        type=str,
        choices=AVAILABLE_GEN_FORMATS,
        default="csv",
        help=("The format of the output (export) values are: ".join(SUPPORTED_FORMATS)),
    )
    parser_calculate.set_defaults(func=command_calculate)  # function command calculate

    return parser



import subprocess

# import tempfile
# import shutil

# from src.config.settings import DEFAULT_CONFIG_FILE_PATH


def capture(command):
    proc = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    out, err = proc.communicate()
    return out, err, proc.returncode


# def test_msgram_list_should_execute_sucessfully():
#     temp_path = tempfile.mkdtemp()

#     _, _, returncode = capture(["msgram", "init", "-cp", DEFAULT_CONFIG_FILE_PATH])

#     assert returncode == 0
#     shutil.rmtree(temp_path)



class MeasureSoftGramCLIException(Exception):
    """Base MeasureSoftGram CLI Exception"""

    pass


class InvalidMetricException(MeasureSoftGramCLIException):
    """Raised when an invalid metric value is provided"""

    pass


class InvalidMetricsJsonFile(MeasureSoftGramCLIException):
    """Raised when the metrics file is invalid"""

    pass


class FileNotFound(MeasureSoftGramCLIException):
    """Raised when a file is not found"""

    pass


class UnableToOpenFile(MeasureSoftGramCLIException):
    """Raised when a file could not be opened"""

    pass


class UnableToReadFile(MeasureSoftGramCLIException):
    """Raised when a file could not be readed"""

    pass


class InitFileAlreadyExists(MeasureSoftGramCLIException):
    """Raised when the init file already exists"""

    pass


class InvalidWeight(MeasureSoftGramCLIException):
    """Raised when a invalid weight is provided to the MeasureSoftGram"""

    pass


class InvalidMeasuresoftgramFormat(MeasureSoftGramCLIException):
    """Raised when a invalid format file is provided to the MeasureSoftGram"""

    pass


class RepositoryUrlNotFound(MeasureSoftGramCLIException):
    """
    Excessão lançada quando não é possível encontrar uma URL que contenha a
    substring do nome do arquivo
    """

    pass


class ConfigFileNotFound(MeasureSoftGramCLIException):
    """Raised when the .measuresoftgram file is not found"""

    pass


class ConfigFileQueryFailed(MeasureSoftGramCLIException):
    """Raised when the query is in the config file"""

    pass


class ConfigFileFormatInvalid(MeasureSoftGramCLIException):
    """Raised when the config file is invalid"""

    pass



import os
from dotenv import load_dotenv
import logging

from src.cli.parsers import create_parser

from src.config.setup_log import config_logger

logger = logging.getLogger("msgram")


def run_cli():
    parser = create_parser()
    raw_args = parser.parse_args()
    command = getattr(raw_args, "command", "help")

    logger.debug(f"cmd  : {command}")
    logger.debug(f"rargs: {raw_args}")

    if not command:
        logger.critical(f"cmd < {command} > not implemented!")
        parser.print_help()
        return

    cmd_func, args = parse_args(raw_args)

    cmd_func(args)  # type: ignore


def parse_args(raw_args):
    func = getattr(raw_args, "func", "help")

    args = vars(raw_args)
    del args["command"]
    del args["func"]

    logger.debug(f"args : {args}")
    logger.debug(f"func : {func}")
    return func, args


def main():
    """Main entry point for MSGram CLI."""

    load_dotenv()
    log_mod = os.getenv("LOG")
    config_logger(log_mod)

    logger.info("Starting MSGram CLI app")
    run_cli()
    logger.info("Done MSGram CLI app")



import logging

from resources import calculate_measures as core_calculate
from src.config.settings import SUPPORTED_MEASURES

from src.cli.resources.metrics import get_metric_value

logger = logging.getLogger("msgram")


def get_measure_value(measures, subchar):
    measures_calculated = []
    for measure in subchar:
        measure_key = measure["key"]
        measures_calculated.append(
            {
                "key": measure_key,
                "value": {m["key"]: m["value"] for m in measures}[measure_key],
                "weight": measure["weight"],
            }
        )

    return measures_calculated


def calculate_measures(
    json_data,
    config: dict = {
        "characteristics": [{"subcharacteristics": [{"measures": [{"key": ""}]}]}]
    },
):
    extracted = get_metric_value(json_data)

    calculate_infos = []
    for measures in SUPPORTED_MEASURES:
        calculate_infos.append(
            {
                "key": list(measures.keys())[0],
                "metrics": [
                    {
                        "key": metric,
                        "value": (
                            [float(value) for value in extracted[metric]]
                            if extracted.get(metric) and extracted[metric]
                            else None
                        ),
                    }
                    for metric in list(measures.values())[0]["metrics"]
                ],
            }
        )
        new_metrics = []
        for measure in calculate_infos:
            if measure.get("metrics", None) and all(
                metric["value"] for metric in measure["metrics"]
            ):
                new_metrics.append(measure)

        calculate_infos = new_metrics

    headers = ["Id", "Name", "Description", "Value", "Created at"]
    return core_calculate({"measures": calculate_infos}, config), headers



from pathlib import Path

import pytest

from src.cli.jsonReader import open_json_file
from src.cli.resources.measure import calculate_measures


def test_calculate_measures():
    json_data = open_json_file(
        Path(
            "tests/unit/data/fga-eps-mds-2022-2-MeasureSoftGram-CLI-01-05-2023-21-40-30-develop-extracted.msgram"
        )
    )

    infos, headers = calculate_measures(json_data)
    assert headers == ["Id", "Name", "Description", "Value", "Created at"]
    assert "measures" in infos

    measure_result = infos.get("measures")
    measure_expected = [
        {"key": "passed_tests", "value": 1.0},
        {"key": "test_builds", "value": 0.9996066627522133},
        {"key": "test_coverage", "value": 0.40234848484848484},
        {"key": "non_complex_file_density", "value": 0.44347274991556906},
        {"key": "commented_file_density", "value": 0.04318181818181818},
        {"key": "duplication_absense", "value": 1.0},
    ]
    for measure_result, measure_expected in zip(measure_result, measure_expected):
        assert measure_result.get("key") == measure_expected.get("key")
        assert pytest.approx(measure_result.get("value")) == measure_expected.get(
            "value"
        )



EXPECTED_SONAR_JSON_COMPONENTS = [
    {
        "id": "AX9GDsKlZuVL7NjXSAZ4",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:tests/__init__.py",
        "name": "__init__.py",
        "qualifier": "UTS",
        "path": "tests/__init__.py",
        "language": "py",
        "measures": [
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "test_errors", "value": "0", "bestValue": True},
            {"metric": "duplicated_lines_density", "value": "0.0", "bestValue": True},
            {"metric": "test_failures", "value": "0", "bestValue": True},
        ],
    },
    {
        "id": "AX9Fg6R0WRlRH47OejaP",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:measuresoftgram/cli.py",
        "name": "cli.py",
        "qualifier": "FIL",
        "path": "measuresoftgram/cli.py",
        "language": "py",
        "measures": [
            {"metric": "complexity", "value": "2"},
            {"metric": "functions", "value": "2"},
            {"metric": "ncloc", "value": "4"},
            {"metric": "coverage", "value": "100.0", "bestValue": True},
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "comment_lines_density", "value": "20.0", "bestValue": False},
            {"metric": "files", "value": "1"},
            {"metric": "test_errors", "value": "0", "bestValue": True},
            {"metric": "duplicated_lines_density", "value": "0.0", "bestValue": True},
            {"metric": "test_failures", "value": "0", "bestValue": True},
        ],
    },
    {
        "id": "AX9GDsKlZuVL7NjXSAZ3",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:tests/hello_world_test.py",
        "name": "hello_world_test.py",
        "qualifier": "UTS",
        "path": "tests/hello_world_test.py",
        "language": "py",
        "measures": [
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "test_execution_time", "value": "2"},
            {"metric": "tests", "value": "2"},
            {"metric": "test_errors", "value": "0", "bestValue": True},
            {"metric": "duplicated_lines_density", "value": "0.0", "bestValue": True},
            {"metric": "test_failures", "value": "0", "bestValue": True},
        ],
    },
    {
        "id": "AX9Fg6R0WRlRH47OejaQ",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:measuresoftgram",
        "name": "measuresoftgram",
        "qualifier": "DIR",
        "path": "measuresoftgram",
        "measures": [
            {"metric": "duplicated_lines_density", "value": "0.0", "bestValue": True},
            {"metric": "functions", "value": "2"},
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "files", "value": "1"},
            {"metric": "complexity", "value": "2"},
            {"metric": "ncloc", "value": "4"},
            {"metric": "coverage", "value": "100.0", "bestValue": True},
            {"metric": "comment_lines_density", "value": "20.0", "bestValue": False},
        ],
    },
    {
        "id": "AX9GDsKlZuVL7NjXSAZ5",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:tests",
        "name": "tests",
        "qualifier": "DIR",
        "path": "tests",
        "measures": [
            {"metric": "test_execution_time", "value": "2"},
            {"metric": "test_failures", "value": "0", "bestValue": True},
            {"metric": "test_errors", "value": "0", "bestValue": True},
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "tests", "value": "2"},
        ],
    },
    {
        "id": "AX9FgyLHNIj_v_uQK41e",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI",
        "name": "2021-2-MeasureSoftGram-CLI",
        "qualifier": "TRK",
        "measures": [
            {"metric": "duplicated_lines_density", "value": "0.0", "bestValue": True},
            {"metric": "functions", "value": "2"},
            {"metric": "test_execution_time", "value": "2"},
            {"metric": "test_failures", "value": "0", "bestValue": True},
            {"metric": "test_errors", "value": "0", "bestValue": True},
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "tests", "value": "2"},
            {"metric": "files", "value": "1"},
            {"metric": "complexity", "value": "2"},
            {"metric": "ncloc", "value": "4"},
            {"metric": "coverage", "value": "100.0", "bestValue": True},
            {"metric": "comment_lines_density", "value": "20.0", "bestValue": False},
        ],
    },
]



import os
import sys
import pytest
import tempfile
import shutil

from io import StringIO
from pathlib import Path
from unittest.mock import patch

from src.cli.commands.cmd_init import command_init

INIT_ARGS = {"config_path": ".testmsgram"}


@pytest.mark.parametrize("init_arg", ["config_path"])
def test_init_invalid_args(init_arg):
    captured_output = StringIO()
    sys.stdout = captured_output

    with pytest.raises(SystemExit):
        command_init({})

    sys.stdout = sys.__stdout__
    assert (
        f"KeyError: args['{init_arg}'] - non-existent parameters"
        in captured_output.getvalue()
    )


def test_init_config_file():
    temp_path = tempfile.mkdtemp()
    config_path = f'{temp_path}/{INIT_ARGS["config_path"]}'

    captured_output = StringIO()
    sys.stdout = captured_output

    command_init({"config_path": Path(config_path)})
    sys.stdout = sys.__stdout__

    assert len(os.listdir(config_path)) == 1
    assert os.listdir(config_path)[0] == "msgram.json"

    assert (
        "The file config: '.testmsgram/msgram.json' was created successfully."
        in captured_output.getvalue()
    )

    shutil.rmtree(temp_path)


def test_init_replace_file():
    config_path = tempfile.mkdtemp()

    captured_output = StringIO()
    sys.stdout = captured_output

    shutil.copy("tests/unit/data/msgram.json", f"{config_path}/msgram.json")

    with patch("builtins.input", return_value="n"):
        command_init({"config_path": Path(config_path)})
        sys.stdout = sys.__stdout__

    assert len(os.listdir(config_path)) == 1
    assert os.listdir(config_path)[0] == "msgram.json"

    assert (
        f"The file config: '{config_path.split('/')[-1]}/msgram.json' not changed..."
        in captured_output.getvalue()
    )

    shutil.rmtree(config_path)



from io import StringIO
from pathlib import Path
import sys
import json
import tempfile
from src.cli.commands.cmd_init import command_init

from src.cli.commands.cmd_list import command_list, print_json_tree

import re

import pytest

INIT_ARGS = {"config_path": ".testmsgram"}


def test_print_json_tree():
    file = open("tests/unit/data/newmsgram.json")
    data = json.load(file)

    captured_output = StringIO()
    sys.stdout = captured_output

    characteristics = data.get("characteristics", [])

    result = print_json_tree(characteristics[0])

    fileExpected = open("tests/unit/data/expected_list.txt")

    compare = fileExpected.read()

    # O padrão de regex para cores no formato [#FFFFFF] e [#458B00]
    color_pattern = r"\[#\w+\]"

    # Substituir todas as ocorrências do padrão pelo texto vazio
    result = re.sub(color_pattern, "", result)
    result = re.sub("\n", "", result)
    compare = re.sub("\n", "", compare)

    assert result == compare


def test_cmd_list():
    temp_path = tempfile.mkdtemp()
    config_path = f'{temp_path}/{INIT_ARGS["config_path"]}'

    captured_output = StringIO()
    sys.stdout = captured_output

    command_init({"config_path": Path(config_path)})

    command_list({"config_path": Path(config_path)})
    sys.stdout = sys.__stdout__

    assert (
        "Para editar o arquivo de configuração utilize em seu terminal o seguinte comando:"
        in captured_output.getvalue()
    )


def test_cmd_list_if_path_not_exists():
    captured_output = StringIO()
    sys.stdout = captured_output

    with pytest.raises(SystemExit):
        command_list({"config_path": Path.cwd() / "invalid_path"})

    sys.stdout = sys.__stdout__

    assert "O arquivo de configuração não foi encontrado." in captured_output.getvalue()



from pathlib import Path

from src.cli.resources.subcharacteristic import calculate_subcharacteristics
from src.cli.jsonReader import open_json_file


def test_calculate_subcharacteristics():
    config = open_json_file(Path("tests/unit/data/msgram.json"))
    measures = [
        {"key": "passed_tests", "value": 1.0},
        {"key": "test_builds", "value": 0.9999969696180555},
        {"key": "test_coverage", "value": 0.5153846153846154},
        {"key": "non_complex_file_density", "value": 0.4829268292682926},
        {"key": "commented_file_density", "value": 0.029230769230769227},
        {"key": "duplication_absense", "value": 1.0},
    ]

    infos, headers = calculate_subcharacteristics(config, measures)

    assert headers == ["Id", "Name", "Description", "Value", "Created at"]
    assert infos == {
        "subcharacteristics": [
            {"key": "testing_status", "value": 0.8633460569923477},
            {"key": "modifiability", "value": 0.650528195701257},
        ]
    }



from rich.console import Console
from src.cli.utils import print_info, print_rule, print_error

from src.config.settings import (
    FILE_CONFIG,
    DEFAULT_CONFIG_PATH,
    DEFAULT_CONFIG_FILE_PATH,
)

from pathlib import Path

import json
import os


def print_json_tree(data):
    result = []
    stack = [(data, "")]
    is_top = True

    measure_to_metric = {}
    measure_to_metric["passed_tests"] = ["tests", "test_failures", "test_errors"]
    measure_to_metric["test_builds"] = ["tests", "test_execution_time"]
    measure_to_metric["test_coverage"] = ["coverage"]
    measure_to_metric["non_complex_file_density"] = ["functions", "complexity"]
    measure_to_metric["commented_file_density"] = ["comment_lines_density"]
    measure_to_metric["duplication_absense"] = ["duplicated_lines_density"]
    measure_to_metric["team_throughput"] = ["resolved_issues", "total_issues"]
    measure_to_metric["ci_feedback_time"] = ["sum_ci_feedback_times", "total_builds"]

    while stack:
        data, indent = stack.pop()
        key = data.get("key")

        if is_top:
            result.append("[#FFFFFF]\nCaracterística:")
            is_top = False
        result.append(f"[#FFFFFF]{indent}[#00FF00]{key}")

        weight = data.get("weight", 0)
        result.append(f"[#FFFFFF]{indent}Peso: [#00FF00]{weight}%")

        if "subcharacteristics" in data:
            for subchar in data["subcharacteristics"]:
                result.append(f"[#FFFFFF]{indent}Sub-característica(s):")
                stack.append(
                    (subchar, f"{indent}│  ")
                )  # Use the ASCII character │ (code 179)

        if "measures" in data:
            for measure in data["measures"]:
                result.append(f"[#FFFFFF]{indent}│  Medida(s):")
                measure_key = measure.get("key")
                result.append(f"[#FFFFFF]{indent}{indent}│  [#00FF00]{measure_key}")
                result.append(
                    f"[#FFFFFF]{indent}{indent}│  Peso: [#00FF00]{measure['weight']}%"
                )
                if "min_threshold" in measure and "max_threshold" in measure:
                    min_threshold = measure.get("min_threshold")
                    max_threshold = measure.get("max_threshold")
                    result.append(f"[#FFFFFF]{indent}{indent}│  Métrica(s):")
                    metrics = measure_to_metric.get(
                        measure_key, []
                    )  # Get associated metrics
                    for metric in metrics:
                        result.append(
                            f"[#FFFFFF]{indent}{indent}│  └─[#00FF00]{metric}"
                        )
                        min_max = f"Min = [#00FF00]{min_threshold} [#FFFFFF]e Max = [#00FF00]{max_threshold}"
                    result.append(
                        f"[#FFFFFF]{indent}{indent}│  │ Valores de referência: {min_max}"
                    )
                    result.append(f"[#FFFFFF]{indent}{indent}│  Fim-Métrica(s)")
                result.append(f"[#FFFFFF]{indent}│  Fim-Medida(s)")
            result.append("[#FFFFFF]Fim-SubCaracterística")
    result.append("[#FFFFFF]Fim-Característica")

    return "\n".join(result)


def command_list(args):
    console = Console()
    console.clear()

    file_path = DEFAULT_CONFIG_FILE_PATH
    try:
        config_path: Path = args["config_path"]

        if config_path != DEFAULT_CONFIG_PATH:
            print_info("[#A9A9A9] Será usado arquivo informado pelo usuário: ")
            file_path = str(config_path) + "/msgram.json"
        else:
            print_info(
                "[#A9A9A9]Não foi informado caminho do arquivo de configuração, será usado caminho padrão."
            )

    except Exception as e:
        print_error(f"KeyError: args[{e}] - non-existent parameters")
        exit(1)

    print_rule("[#FFFFFF]Listing Configuration Parameters")

    if not (os.path.exists(file_path)):
        print_info("[#A9A9A9] O arquivo de configuração não foi encontrado. \n")
        print_info(
            "Execute o comando 'msgram init' no projeto desejado para criá-lo.\n"
        )
        print_info(
            "Ou use 'msgram init --config_path <path>' para informar o caminho ate o arquivo."
        )
        exit()

    print_info(f"MSGram config file [bold red]'{FILE_CONFIG}'[/] exists already!")

    f = open(file_path)

    data = json.load(f)

    for characteristic in data.get("characteristics", []):
        output_string = print_json_tree(characteristic)
        print_info(output_string)

    print_info(
        "\n[#A9A9A9]Para editar o arquivo de configuração utilize em seu terminal o seguinte comando:"
    )
    print_info("vim .msgram/msgram.json\n")



import subprocess
import tempfile
import shutil


def capture(command):
    proc = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    out, err = proc.communicate()
    return out, err, proc.returncode


def test_msgram_init_should_execute_sucessfully():
    temp_path = tempfile.mkdtemp()
    dir_path = f"{temp_path}/.msgram"

    _, _, returncode = capture(["msgram", "init", "-cp", dir_path])

    assert returncode == 0
    shutil.rmtree(temp_path)


def test_init_should_create_pre_config_in_existent_directory():
    temp_path = tempfile.mkdtemp()

    _, err, returncode = capture(["msgram", "init", "-cp", temp_path])

    assert returncode == 0
    shutil.rmtree(temp_path)



from datetime import datetime
import json
import logging
import os
import re
import sys
from time import perf_counter

from rich import print
from rich.console import Console
from genericparser import GenericParser

from src.cli.jsonReader import folder_reader
from src.cli.utils import (
    make_progress_bar,
    print_info,
    print_panel,
    print_rule,
    print_warn,
)

logger = logging.getLogger("msgram")


def get_infos_from_name(filename: str) -> str:
    """
    filename: str = fga-eps-mds-2022-1-MeasureSoftGram-Service-09-11-2022-16-11-42-develop.json
    """
    file_date = re.search(r"\d{1,2}-\d{1,2}-\d{4}-\d{1,2}-\d{1,2}", filename)

    if not file_date:
        message = (
            "Could not extract creation date from file. Was the file name "
            "to contain a date in the format dd-mm-yyyy-hh-mm"
        )
        print_warn(message)
        print_warn(f"filename: {filename}")
        sys.exit(1)

    file_name = filename.split(".")[0]

    return f"{file_name}-extracted.msgram"


def command_extract(args):
    time_init = perf_counter()
    try:
        output_origin = args["output_origin"]
        extracted_path = args["extracted_path"]
        data_path = args.get("data_path", None)
        language_extension = args["language_extension"]
        repository_path = args.get("repository_path", None)

    except Exception as e:
        logger.error(f"KeyError: args[{e}] - non-existent parameters")
        print_warn(f"KeyError: args[{e}] - non-existent parameters")
        exit(1)

    if data_path is None and repository_path is None:
        logger.error(
            "It is necessary to pass the data_path or repository_path parameters"
        )
        print_warn(
            "It is necessary to pass the data_path or repository_path parameters"
        )
        sys.exit(1)

    console = Console()
    console.clear()
    print_rule("Extract metrics")
    parser = GenericParser()

    if repository_path and output_origin == "github":
        result = parser.parse(input_value=repository_path, type_input=output_origin)
        repository_name = repository_path.replace("/", "-")
        save_file_with_results(
            ".msgram",
            repository_path,
            name=f"github_{repository_name}-{datetime.now().strftime('%d-%m-%Y-%H-%M-%S')}-extracted.msgram",
            result=result,
        )
        return

    if not os.path.isdir(extracted_path):
        logger.error(
            f'FileNotFoundError: extract directory "{extracted_path}" does not exists'
        )
        print_warn(
            f"FileNotFoundError: extract directory[blue]'{extracted_path}'[/]does not exists"
        )
        sys.exit(1)

    logger.debug(f"output_origin: {output_origin}")
    logger.debug(f"data_path: {data_path}")
    logger.debug(f"language_extension: {language_extension}")
    logger.debug(f"extracted_path: {extracted_path}")

    files = list(data_path.glob("*.json"))
    valid_files = len(files)

    print_info(f"\n> Extract and save metrics [[blue ]{output_origin}[/]]:")
    with make_progress_bar() as progress_bar:
        task_request = progress_bar.add_task(
            "[#A9A9A9]Extracting files: ", total=len(files)
        )
        progress_bar.advance(task_request)

        for component, filename, files_error in folder_reader(data_path, "json"):
            if files_error:
                progress_bar.update(task_request, advance=files_error)
                valid_files = valid_files - files_error

            name = get_infos_from_name(filename)
            result = parser.parse(input_value=component, type_input=output_origin)

            save_file_with_results(extracted_path, filename, name, result)

            progress_bar.advance(task_request)

        time_extract = perf_counter() - time_init
        print_info(
            f"\n\nMetrics successfully extracted [[blue bold]{valid_files}/{len(files)} "
            f"files - {time_extract:0.2f} seconds[/]]!"
        )
    print_panel(
        "> Run [#008080]msgram calculate all -ep 'extracted_path' -cp 'extracted_path' -o 'output_origin'"
    )


def save_file_with_results(extracted_path, filename, name, result):
    print(f"[dark_green]Reading:[/] [black]{filename}[/]")
    print(f"[dark_green]Save   :[/] [black]{name}[/]\n")

    with open(f"{extracted_path}/{name}", "w") as f:
        f.write(json.dumps(result, indent=4))



from src.cli.resources.characteristic import get_characteristic_value

from resources import calculate_tsqmi as core_calculate


def calculate_tsqmi(config, characteristics):
    tsqmi = config
    calculate_infos = {
        "key": "tsqmi",
        "characteristics": get_characteristic_value(
            characteristics, tsqmi["characteristics"]
        ),
    }

    headers = ["Id", "Value", "Created at"]
    return core_calculate({"tsqmi": calculate_infos}), headers



import csv
import json
import logging
import re
from pathlib import Path

from anytree import Node, RenderTree

from rich import print
from rich.console import Console
from staticfiles import DEFAULT_PRE_CONFIG as pre_config

from src.cli.jsonReader import open_json_file, read_multiple_files
from src.cli.resources.characteristic import calculate_characteristics
from src.cli.resources.measure import calculate_measures
from src.cli.resources.tsqmi import calculate_tsqmi
from src.cli.resources.subcharacteristic import calculate_subcharacteristics
from src.cli.utils import print_error, print_info, print_panel, print_rule, print_table
from src.cli.aggregate_metrics import aggregate_metrics
from src.cli.exceptions import exceptions
from src.config.settings import DEFAULT_CONFIG_PATH, FILE_CONFIG

logger = logging.getLogger("msgram")


def read_config_file(config_path):
    try:
        return open_json_file(config_path / FILE_CONFIG)
    except exceptions.MeasureSoftGramCLIException as e:
        print_error(
            f"[red]Error reading msgram.json config file in {config_path}: {e}\n"
        )
        print_rule()
        exit(1)


def calculate_metrics(extracted_path, config):
    data_calculated = []

    if not extracted_path.is_file():
        if not aggregate_metrics(extracted_path, config):
            print_error(
                "> [red] Failed to aggregate metrics, calculate was not performed. \n"
            )
            return data_calculated, False

        for file, file_name in read_multiple_files(extracted_path, "metrics"):
            result = calculate_all(file, file_name, config)
            data_calculated.append(result)

        return data_calculated, True
    else:
        try:
            result = calculate_all(
                open_json_file(extracted_path), extracted_path.name, config
            )
            return result, True
        except exceptions.MeasureSoftGramCLIException as e:
            print_error(f"[red]Error calculating {extracted_path}: {e}\n")
            return data_calculated, False


def command_calculate(args):
    try:
        output_format: str = args["output_format"]
        config_path = args["config_path"]
        extracted_path = args["extracted_path"]

    except KeyError as e:
        logger.error(f"KeyError: args[{e}] - non-existent parameters")
        print_error(f"KeyError: args[{e}] - non-existent parameters")
        exit(1)

    console = Console()
    console.clear()
    print_rule("Calculate")
    print_info("> [blue] Reading config file:[/]")

    config = read_config_file(config_path)

    print_info("\n> [blue] Reading extracted files:[/]")

    data_calculated, success = calculate_metrics(extracted_path, config)

    if success:
        print_info("\n[#A9A9A9]All calculations performed[/] successfully!")

    show_results(output_format, data_calculated, config_path)
    print_rule()

    print_panel(
        title="Done",
        menssage="> See our docs for more information: \n"
        " https://github.com/fga-eps-mds/2022-2-MeasureSoftGram-CLI",
    )


def calculate_all(json_data, file_name, config):
    data_measures, _ = calculate_measures(json_data, config)
    data_subcharacteristics, _ = calculate_subcharacteristics(
        config, data_measures["measures"]
    )
    data_characteristics, _ = (
        calculate_characteristics(config, data_subcharacteristics["subcharacteristics"])
        if data_subcharacteristics["subcharacteristics"]
        else ([], [])
    )
    data_tsqmi, _ = (
        calculate_tsqmi(config, data_characteristics["characteristics"])
        if data_characteristics
        else ([], [])
    )

    version = re.search(r"\d{1,2}-\d{1,2}-\d{4}-\d{1,2}-\d{1,2}", file_name)[0]
    repository = file_name.split(version)[0][:-1]

    return {
        "repository": [{"key": "repository", "value": repository}],
        "version": [{"key": "version", "value": version}] if version else [],
        "measures": data_measures["measures"] if data_measures else [],
        "subcharacteristics": data_subcharacteristics["subcharacteristics"]
        if data_subcharacteristics
        else [],
        "characteristics": data_characteristics["characteristics"]
        if data_characteristics
        else [],
        "tsqmi": data_tsqmi["tsqmi"] if data_tsqmi else [],
    }


def show_results(output_format, data_calculated, config_path):

    if output_format == "tabular":
        show_tabulate(data_calculated)

    elif output_format == "raw":
        print(data_calculated)

    elif output_format == "tree":
        show_tree(data_calculated, pre_config)

    elif len(data_calculated) == 0:
        print_info(
            f"[yellow]WARNING: No extracted file read so no {output_format} was generated!"
        )

    elif output_format == "csv":
        print_info("Exporting CSV...")
        export_csv(data_calculated, config_path)

    elif output_format == "json":
        print_info("Exporting JSON...")
        export_json(data_calculated, config_path)


def show_tabulate(data_calculated):
    tsqmi = data_calculated["tsqmi"][0]
    characteristics = {c["key"]: c["value"] for c in data_calculated["characteristics"]}
    subcharacteristics = {
        sc["key"]: sc["value"] for sc in data_calculated["subcharacteristics"]
    }
    measures = {m["key"]: m["value"] for m in data_calculated["measures"]}

    print_table(measures, "measures", "measures")
    print_table(subcharacteristics, "subcharacteristics", "subcharacteristics")
    print_table(characteristics, "characteristics", "characteristics")
    print_table(tsqmi, "tsqmi", "tsqmi")


def get_obj_by_element(object_list: list, element_key: str, element_to_find):
    return next((obj for obj in object_list if obj[element_key] == element_to_find), {})


def show_tree(data_calculated, pre_config):
    tsqmi = data_calculated["tsqmi"][0]
    characteristics = data_calculated["characteristics"]
    subcharacteristics = data_calculated["subcharacteristics"]
    measures = data_calculated["measures"]

    print("Overview - tree:\n\n")
    tsqmi_tree = Node(f"[green]{tsqmi['key']}: {tsqmi['value']}")

    for char_c, char in zip(pre_config["characteristics"], characteristics):
        char_tree = Node(f"[red]{char['key']}: {char['value']}", parent=tsqmi_tree)

        for subchar_c in char_c["subcharacteristics"]:
            subchar = get_obj_by_element(subcharacteristics, "key", subchar_c["key"])
            if subchar:
                sub_char_tree = Node(f"[blue]{subchar['key']} {subchar['value']}", parent=char_tree)

                for measure_c in subchar_c["measures"]:
                    measure = get_obj_by_element(measures, "key", measure_c["key"])
                    if measure:
                        Node(f"[yellow]{measure['key']} {measure['value']}", parent=sub_char_tree)

    for pre, fill, node in RenderTree(tsqmi_tree):
        print(f"{pre}{node.name}")


def export_json(data_calculated: list, file_path: Path = DEFAULT_CONFIG_PATH):
    file_path = file_path.joinpath("calc_msgram.json")
    with open(file_path, "w", encoding="utf-8") as write_file:
        json.dump(
            data_calculated,
            write_file,
            indent=4,
        )
    print_info(f"[blue]Success:[/] {file_path.name} [blue]exported as JSON")


def export_csv(data_calculated: list, file_path: Path = Path("DEFAULT_CONFIG_PATH")):
    file_path = file_path.joinpath("calc_msgram.csv")

    with open(file_path, "w", newline="") as csv_file:
        writer = csv.writer(csv_file)

        csv_header = []
        csv_rows = []

        for item in data_calculated:
            if isinstance(item, dict):
                header_column = []
                columns = []

                for _, value in item.items():
                    for column in value:
                        header_column.append(column["key"])
                        columns.append(column["value"])

                csv_header.extend(header_column)
                csv_rows.append(columns)

        writer.writerow(csv_header)
        writer.writerows(csv_rows)

    print(f"Success: {file_path.name} exported as CSV")



from pathlib import Path

FILE_CONFIG = "msgram.json"
DEFAULT_CONFIG_PATH = Path.cwd() / ".msgram"
DEFAULT_RAW_DATA_PATH = Path.cwd() / "analytics-raw-data"
DEFAULT_CONFIG_FILE_PATH = DEFAULT_CONFIG_PATH / FILE_CONFIG

AVAILABLE_ENTITIES = [
    "metrics",
    "measures",
    "subcharacteristics",
    "characteristics",
    "tsqmi",
]

SUPPORTED_FORMATS = [
    "json",
    "tabular",
]

AVAILABLE_IMPORTS = ["sonarqube", "github"]

AVAILABLE_GEN_FORMATS = ["csv", "json"]

SUPPORTED_MEASURES = [
    {
        "passed_tests": {
            "metrics": [
                "tests",
                "test_failures",
                "test_errors",
            ],
        }
    },
    {
        "test_builds": {
            "metrics": [
                "test_execution_time",
                "tests",
            ],
        }
    },
    {
        "test_coverage": {
            "metrics": [
                "coverage",
            ],
        }
    },
    {
        "non_complex_file_density": {
            "metrics": [
                "functions",
                "complexity",
            ],
        }
    },
    {
        "commented_file_density": {
            "metrics": [
                "comment_lines_density",
            ],
        }
    },
    {
        "duplication_absense": {
            "metrics": [
                "duplicated_lines_density",
            ],
        }
    },
    {
        "team_throughput": {
            "metrics": [
                "total_issues",
                "resolved_issues",
            ],
        }
    },
    {
        "ci_feedback_time": {
            "metrics": [
                "sum_ci_feedback_times",
                "total_builds",
            ],
        }
    },
]
